#define  INITGUID

#include "windows.h"
#include "shared.h"
#include "dinput.h"
#include "input.h"
#include "registry.h"

static LPDIRECTINPUT							pDirectInput	 = NULL;
static LPDIRECTINPUTDEVICE						pKeyboard		 = NULL;
static BOOL										KeyboardAcquired = FALSE;
static unsigned char							KeyMap[256];

static char										ButtonString[256][16] = {"", "Esc", "1", "2", "3", "4", "5", "6", "7", "8",
																		 "9", "0", "-", "=", "Backspace", "Tab", "Q", "W", "E", "R",
																		 "T", "Y", "U", "I", "O", "P", "(", ")", "Return", "L Control",        
																		 "A", "S", "D", "F", "G", "H", "J", "K", "L", ";",
																		 "'", "#", "L Shift", "\\", "Z", "X", "C", "V", "B", "N",
																		 "M", ",", ".", "/", "R Shift", "*", "L Menu", "Space", "Caps", "F1",
																		 "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10", "Num Lock",          
																		 "Scroll Lock", "Num 7", "Num 8", "Num 9", "Num -", "Num 4", "Num 5", "Num 6", "Num +", "Num 1",
																		 "Num 2", "Num 3", "Num 0", "Num .", "", "", "OEM 102", "F11", "F12", "", "", "",
																		 "", "", "", "", "", "", "", "", "F13", "F14",
																		 "F15", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
																		 "","", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
																		 "", "", "", "", "Num =", "", "", "", "", ":",
																		 "_", "", "", "", "", "", "", "", "", "Enter", "R Control", "", "", "", "", "", "", "",
																		 "", "", "", "", "", "", "", "", "", "", "", "", "", "", "Num ,", "", "Num /", "", "Prt Sc", "R Menu", "", "", "",
																		 "", "", "", "", "", "", "", "", "", "Pause", "", 
																		 "Home", "Up", "Page Up", "", "Left", "", "Right", "", "End", "Down", "Page Down", "Insert", "Delete",
																		 "", "", "", "", "", "", "", "L Win", "R Win"};
	
BOOL InitInput(HWND hwnd, HINSTANCE hInstance)
{
	GUID		guid = GUID_SysKeyboard;

	if (DirectInputCreate(hInstance, 0x0300, &pDirectInput, NULL) != DI_OK)
	{
		TidyInput();
		return FALSE;
	}

	if (IDirectInput_CreateDevice(pDirectInput, &guid, &pKeyboard, NULL) != DI_OK)
	{
		TidyInput();
		return FALSE;
	}

	if (IDirectInputDevice_SetDataFormat(pKeyboard, &c_dfDIKeyboard) != DI_OK)
	{
		TidyInput();
		return FALSE;
	}

    if (IDirectInputDevice_SetCooperativeLevel(pKeyboard, hwnd, DISCL_NONEXCLUSIVE | DISCL_FOREGROUND) != DI_OK)
	{
		TidyInput();
		return FALSE;
	}

	AcquireKeyboard();

	return TRUE;
}

void TidyInput(void)
{
    if (pKeyboard)
	{
	    UnacquireKeyboard();
	    IDirectInputDevice_Release(pKeyboard);
		pKeyboard = NULL;
	}

    if (pDirectInput)
    {
        IDirectInputDevice_Release(pDirectInput);
		pDirectInput = NULL;
    }
}

void AcquireKeyboard(void)
{
	if (pKeyboard)
	{	       
		KeyboardAcquired = FALSE;

		if(IDirectInputDevice_Acquire(pKeyboard) == DI_OK)
		{
			KeyboardAcquired = TRUE;
		}

		memset(KeyMap, 0, sizeof(char) * 256);
	}
}

void UnacquireKeyboard(void)
{
	if (pKeyboard)
	{
		if (KeyboardAcquired)
		{
			if (IDirectInputDevice_Unacquire(pKeyboard) == DI_OK)
			{
				KeyboardAcquired = FALSE;
			}
		}
	}
}

void UpdateKeyboard(void)
{
	HRESULT Result;

	if (pKeyboard)
	{
	    Result = IDirectInputDevice_GetDeviceState(pKeyboard, sizeof(KeyMap), KeyMap);

	    if ((Result == DIERR_INPUTLOST) || (Result == DIERR_NOTACQUIRED))
	    {
			AcquireKeyboard();
		}
	}

	input_mask = 0;

	if (KeyMap[RegistryInfo.ButtonMap[PLAYER1_LEFT]])		input_mask |= KEY_PAD1_LEFT;
	if (KeyMap[RegistryInfo.ButtonMap[PLAYER1_RIGHT]])		input_mask |= KEY_PAD1_RIGHT;
	if (KeyMap[RegistryInfo.ButtonMap[PLAYER1_UP]])			input_mask |= KEY_PAD1_UP;
	if (KeyMap[RegistryInfo.ButtonMap[PLAYER1_DOWN]])		input_mask |= KEY_PAD1_DOWN;
	if (KeyMap[RegistryInfo.ButtonMap[PLAYER1_BUTTON1]])	input_mask |= KEY_PAD1_BUTTON1;
	if (KeyMap[RegistryInfo.ButtonMap[PLAYER1_BUTTON2]])	input_mask |= KEY_PAD1_BUTTON2;
	if (KeyMap[RegistryInfo.ButtonMap[PLAYER1_START]])		input_mask |= (cart.type)? KEY_PAD_START : KEY_PAD_PAUSE;

	if (KeyMap[RegistryInfo.ButtonMap[PLAYER2_LEFT]])		input_mask |= KEY_PAD2_LEFT;
	if (KeyMap[RegistryInfo.ButtonMap[PLAYER2_RIGHT]])		input_mask |= KEY_PAD2_RIGHT;
	if (KeyMap[RegistryInfo.ButtonMap[PLAYER2_UP]])			input_mask |= KEY_PAD2_UP;
	if (KeyMap[RegistryInfo.ButtonMap[PLAYER2_DOWN]])		input_mask |= KEY_PAD2_DOWN;
	if (KeyMap[RegistryInfo.ButtonMap[PLAYER2_BUTTON1]])	input_mask |= KEY_PAD2_BUTTON1;
	if (KeyMap[RegistryInfo.ButtonMap[PLAYER2_BUTTON2]])	input_mask |= KEY_PAD2_BUTTON2;
	if (KeyMap[RegistryInfo.ButtonMap[PLAYER2_START]])		input_mask |= (cart.type)? KEY_PAD_START : KEY_PAD_PAUSE;
}

void SetControlKey(int Button, int Key)
{
	RegistryInfo.ButtonMap[Button] = Key;
}

char *ButtonToString(int Button)
{
	return ButtonString[RegistryInfo.ButtonMap[Button]];
}

int GetKeyPressed(void)
{
	int			i;

	UpdateKeyboard();

	for (i = 0 ; i < 256 ; i++)
	{
		if (KeyMap[i]) return i;
	}

	return -1;
}